# -*- coding: utf-8; Mode: Python; indent-tabs-mode: nil; tab-width: 4 -*-

# Copyright (C) 2014 nfschina.
# Written by Li Ting <liting@nfschina.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from __future__ import print_function

import os
import subprocess
import sys

from ubiquity import i18n, misc, osextras, plugin, validation


NAME = 'overview'
AFTER = 'alternative'
WEIGHT = 11
OEM = False


# TODO: This cannot be a non-debconf plugin after all as OEMs may want to
# preseed the 'install updates' and 'install non-free software' options.  So?
# Just db_get them.  No need for any other overhead, surely.  Actually, you
# need the dbfilter for that get.



class PageGtk(plugin.PluginUI):

    plugin_title = 'ubiquity/text/overview_heading_label'
    help_dialog = 'help_overview'
    help_label = 'overview_text_label'
    help_prompt = 'prompt_overview'

    def __init__(self, controller, *args, **kwargs):
        self.controller = controller
        from gi.repository import Gtk
        builder = Gtk.Builder()
        self.controller.add_builder(builder)
        builder.add_from_file(os.path.join(
            os.environ['UBIQUITY_GLADE'], 'stepOverview.ui'))
        builder.connect_signals(self)
        self.cancel_num = 0 
        self.page = builder.get_object('stepOverview')
#        self.disksize = builder.get_object('disksize')
#        self.fstype = builder.get_object('fstype')
#        self.swappartition = builder.get_object('swappartition')
#        self.swapsize = builder.get_object('swapsize')
        self.overview_treeview = builder.get_object('overview_treeview')
        self.timezoneinfodict={'Asia/Beijing':'北京标准时间',
                               'Asia/Chongqing':'澳门时间',
                               'Asia/Urumuqi':'乌鲁木齐时间',
                               'Asia/Hong_Kong':'香港时间',
                               'Asia/Taipei':'台湾时间'}
        self.uninstallall= {'kingsoft-office': 'WPS办公软件',
                            'evince': 'PDF阅读器',
                            'dia': '工程绘图',
                            'gedit': 'Gedit编辑器',
                            'file-compress': '压缩工具',
                            'iptux': '局域网通讯程序',
                            'nfs-screenshot': '截图工具',
                            'tomboy': '笔记便签',
                            'gnome-calculator': '计算器',
                            'firefox': 'Firefox网络浏览器',
                            'thunderbird': '邮件/新闻',
                            'nfs-firewall': '防火墙设定',
                            'drivercenter': '驱动管理',
                            'filezilla': 'ftp 客户端',
                            'brasero': '光盘刻录机',
                            'gimp': '图片编辑器',
                            'totem': '视频播放器',
                            'rhythmbox': '音乐播放器',
                            'audio-recorder': '音频录制器',
                            'gparted': 'GParted磁盘分区工具',
                            'mintbackup': '备份工具',
                            'vino': '远程桌面'}

        self.plugin_widgets = self.page
#        paddingbox = builder.get_object('paddingbox')

#        def func(box):
#            box.get_parent().child_set_property(box, 'expand', false)
#            box.set_size_request(box.get_allocation().width / 2, -1)

#        paddingbox.connect('realize', func)


    def set_modelcode(self,modelcode):
        if modelcode:
            self.modelcode.set_text(modelcode)

    def get_modelcode(self):
        return self.modelcode.get_text()
 
    def set_layoutcode(self,layoutcode):
        if layoutcode:
            self.layoutcode.set_text(layoutcode)

    def get_layoutcode(self):
        return self.layoutcode.get_text()

    def set_language_info(self, langinfo):
        if langinfo:
            self.linfo.set_text(langinfo)

    def get_language_info(self):
        return self.linfo.get_text()

    def set_username_info(self, usernameinfo):
        if usernameinfo:
            self.userinfo.set_text(usernameinfo)

    def get_username_info(self):
        return self.userinfo.get_text()

    def set_passwd_info(self, passwordinfo):
        if passwordinfo:
            self.passwdinfo.set_text(passwordinfo)
    
    def get_passwd_info(self):
        return self.passwdinfo.get_text()

    def set_timezone_info(self,timezoneinfo):
        if timezoneinfo:
            self.diskinfo.set_text(timezoneinfo)

    def get_timezone_info(self):
        return self.diskinfo.get_text()

    def get_langreset_info(self):
        return self.langreset.get_active_text()

    def set_keyboard_info(self, keyboardinfo):
        if keyboardinfo:
            self.keyboardinfo.set_text(keyboardinfo)

    def set_partition_info(self, partitioninfo):
        if partitioninfo:
            self.partitioninfo.set_text(partitioninfo)
    
    def set_disksize_info(self, disksizeinfo):
        if disksizeinfo:
            self.disksize.set_text(disksizeinfo)

    def set_fstype_info(self, filesystemtype):
        if filesystemtype:
            self.fstype.set_text(filesystemtype)

    def set_swap_info(self, devpart, swapsizetext):
        if devpart and swapsizetext:
            self.swappartition.set_text(devpart)
            self.swapsize.set_text(swapsizetext)

    def plugin_on_help_clicked(self):
        self.show_help('/usr/share/ubiquity/gtk/stepHelp.ui', self.help_dialog, self.help_label, self.help_prompt)

    def on_inforeset_clicked(self,ubused_widget):
        resetlang=self.get_langreset_info() 
        resettz=self.get_timezone_info()
        resetusername=self.get_username_info()
        resetpasswd=self.get_passwd_info
        self.set_language_info(resetlang)         
        self.verifylabel.hide()
        self.verifylabel.show()

    def set_overview_treeview(self, infodict):
        from gi.repository import Gtk

        treestore = self.overview_treeview.get_model()
        if treestore is None:
            treestore=Gtk.TreeStore(str)
            renderer = Gtk.CellRendererText()
            column = Gtk.TreeViewColumn(self.controller.get_string('overview_column'),renderer,text=0, foreground =3)
            self.overview_treeview.append_column(column)
            self.overview_treeview.set_model(treestore)
        else:
            treestore.clear()
        if infodict:
            for key,value in infodict.items():
                if key != '分区设置：' and key != 'Partition Setting:' and key != '卸载应用：' and key != 'Uninstalled:':
                    piter = treestore.append(None,[key])
                    treestore.append(piter, [value])
                else:
                    if len(value) == 1:
                        piter = treestore.append(None,[key])
                        treestore.append(piter, value)
                    else:
                        piter = treestore.append(None,[key])
                        for item in range(len(value)):
                            treestore.append(piter, [value[item]])

        self.overview_treeview.expand_all()

# add this function to get partition information when replace exist os
# parition_cache has nothing in this case
# we should gei partition info from other place
    def get_devpart(self):
        import subprocess
        import locale

        devparts = devsizes = fstype = 'N/A'
        encoding = locale.getdefaultlocale()[1]

        cmddf = subprocess.Popen(["df", "-h"], stdout=subprocess.PIPE).stdout.read()
        for line in cmddf.decode(encoding).split('\n'):
            columns = line.split()
            if columns:
                devpart = columns[0]
                devsize = columns[1]
                mountpoint = columns[-1]
                if devpart.startswith('/') and 'target' in mountpoint:
                    devparts = devpart
                    devsizes = devsize
        cmdmount = subprocess.Popen(["blkid", devparts], stdout=subprocess.PIPE).stdout.read()
        for line in cmdmount.decode(encoding).split('\n'):
            columnss = line.split()
            if columnss:
                fstype = columnss[-1].split("=")[-1]
        return (devparts, devsizes, fstype)

# end for replace getting
    def get_human_keyboardlayout(self, keyboardinfo):
        import locale
        try:
            locale.setlocale(locale.LC_ALL, '')
        except locale.Error:
            try:
                if os.path.exists("/etc/locale.gen"):
                    with open("/etc/locale.gen", "r") as locale_gen:
                        locale.setlocale(locale.LC_ALL, locale_gen.read().split()[0])
                else:
                    locale.setlocale(locale.LC_ALL, 'en_US.UTF-8')
                human_layout=locale.strxfrm(keyboardinfo.encode('utf-8'))
            except:
                pass
        return human_layout

    def get_human_alternative(self,deleteselect):
        liststr=deleteselect.split(' ')
        selectinfo=[]
        selectinfoeng=[]
        for key,value in self.uninstallall.items():
            for item in range(len(liststr)):
                if key == liststr[item]:
                    selectinfo.append(value)
                    selectinfoeng.append(key)
        return (selectinfo,selectinfoeng)
            

    def get_human_timezoneinfo(self,timezone):
        timezoneinfo=''
        for key ,value in self.timezoneinfodict.items():
            if key == str(timezone):
                timezoneinfoeng = key
                timezoneinfo = value
        if timezoneinfo == '':
            timezoneinfoeng = str(timezone)
            timezoneinfo = str(timezone)
        return (timezoneinfo, timezoneinfoeng)        
#    def on_installbutton_clicked(self, unused_widget):
#        self.verify_dialog.hide()

#    def on_cancebutton_clicked(self, unused_widget):
#        self.controller.allow_go_forward(False)
#        self.verify_dialog.hide()
# test
#    def plugin_on_next_clicked(self):
#        from gi.repository import Gtk
#        self.verify_dialog.show_all()
#        response = self.verify_dialog.run()
#        self.verify_dialog.hide()

#        if response == Gtk.ResponseType.OK:
#            print("ok button click")
#            return True
#        elif response == Gtk.ResponseType.CANCEL:
#            return False
        

class Page(plugin.Plugin):
    def prepare(self):
        usernames = self.db.get('passwd/username')
        langinfo = self.db.get('localechooser/languagelist')
        passwordinfo = self.db.get('passwd/user-password')
        timezoneinfodb = self.db.get('time/zone')
        keyboardmodel = self.db.get('keyboard-configuration/modelcode')
        keyboardlayout = self.db.get('keyboard-configuration/layoutcode')
        partitionnum = self.db.get('ubiquity/partitionnum_liting')
        partitionsize = self.db.get('ubiquity/partitionsize_liting')
        partitionfstype = self.db.get('ubiquity/partitionfstype_liting')
        partitionswappart = self.db.get('ubiquity/partitionswappart_liting')
        partitionswapsize = self.db.get('ubiquity/partitionswapsize_liting')
        login_auto = self.db.get('passwd/auto-login')
        encrypt_home = self.db.get('user-setup/encrypt-home')
        deleteselect = self.db.get('ubiquity/user_alternative')

        if not partitionnum:
            (partitionnum, partitionsize,partitionfstype) = self.ui.get_devpart()


        q='ubiquity/text/partitionclean'
        self.db.subst(q,'partitionnum',partitionnum)
        self.db.subst(q,'partitionsize',partitionsize)
        self.db.subst(q,'partitionfstype',partitionfstype)
        mountdiskinfo=self.extended_description(q)

        if partitionswappart:
            q='ubiquity/text/swappartclean'
            self.db.subst(q,'partitionswappart',partitionswappart)
            self.db.subst(q,'partitionswapsize',partitionswapsize)
            swappartinfo = self.extended_description(q)
        else:
 #           swappartinfo = self.ui.controller.get_string('targetdfswap')
            swappartinfo = '未设置交换分区'

        q='ubiquity/text/userinfoclean'
        self.db.subst(q,'usernames',usernames)
        userinfo = self.extended_description(q)
   
        language = "中文(简体)"
        languageeng = "English"
        if not timezoneinfodb:
            timezoneinfo = '北京标准时间'
            timezoneinfoeng = 'Asia/Beijing'
        else:
            (timezoneinfo, timezoneinfoeng) = self.ui.get_human_timezoneinfo(timezoneinfodb)
# OoooooooOoooooooooooooo, THESE SHOULD BE UPDATED!!!
#        timezoneinfo='中国北京'
#        timezoneinfoeng='Asia/Beijing'
        if str(keyboardlayout) == 'cn':
            keyboardindo = '汉语键盘'
            keyboardindoeng = 'Chinese Keyboard'
        elif str(keyboardlayout) == 'us':
            keyboardindo = '英语键盘'
            keyboardindoeng = 'English Keyboard layout'
        else:
            keyboardindo = '其他键盘'
            keyboardindoeng = 'Other keyboard layout'
        if deleteselect=='':
            deletesoft=['您当前未选择卸载应用']
            deletesofteng=['You select all softwares to be installed.']
        else:
            (deletesoft,deletesofteng)=self.ui.get_human_alternative(deleteselect)
        if langinfo == 'zh_CN':
            treedict={'语言设置：':language,
                      '时间设置：':timezoneinfo,
                      '用户设置：':userinfo,
                      '键盘设置：':keyboardindo,
                      '分区设置：':[mountdiskinfo, swappartinfo],
                      '卸载应用：':deletesoft}
            if str(login_auto) != 'true' and str(encrypt_home) != 'true':
                treedict['登录设置：']= '您已设置用户密码登录'
            elif str(login_auto) == 'true':
                treedict['登录设置：']= '您已设置自动登录'
            else:
                treedict['登录设置：']= '您已设置加密主目录,需密码登录'
        else:
            treedict={'Language setting:':languageeng,
                      'Time Zone Setting:':timezoneinfoeng,
                      'User Information:':userinfo,
                      'Keyboard Setting:':keyboardindoeng,
                      'Partition Setting:':[mountdiskinfo, swappartinfo],
                      'Uninstalled:':deletesofteng}
            if str(login_auto) != 'true' and str(encrypt_home) != 'true':
                treedict['Login Setting:']= 'You have set up login with password'
            elif str(login_auto) == 'true':
                treedict['Login Setting:']= 'You have set up auto-login'
            else:
                treedict['Login Setting:']= 'You have encrypted your home directory'


        self.ui.set_overview_treeview(treedict)

        command = ['/usr/share/ubiquity/simple-plugins', 'overview']
        questions = ['ubiquity/overview_checked']
        return command, questions

    def ok_handler(self):
        self.preseed_bool('ubiquity/overview_checked', True)
        plugin.Plugin.ok_handler(self)
