# -*- coding: utf-8; Mode: Python; indent-tabs-mode: nil; tab-width: 4 -*-

# Copyright (C) 2014 nfschina.
# Written by Yuanzhe <yuanzhe@iscas.ac.cn>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

from __future__ import print_function

import os
import subprocess
import sys

from ubiquity import i18n, misc, osextras, plugin, validation


NAME = 'protocol'
AFTER = 'language'
WEIGHT = 11


# TODO: This cannot be a non-debconf plugin after all as OEMs may want to
# preseed the 'install updates' and 'install non-free software' options. So?
# Just db_get them. No need for any other overhead, surely. Actually, you
# need the dbfilter for that get.



class PageGtk(plugin.PluginUI):

    plugin_title = 'ubiquity/text/protocol_heading_label'
    help_dialog = 'help_protocol'
    help_label = 'protocol_text_label'
    help_prompt = 'prompt_protocol'

    def __init__(self, controller, *args, **kwargs):
        self.controller = controller
        from gi.repository import Gtk
        builder = Gtk.Builder()
        self.controller.add_builder(builder)
        builder.add_from_file(os.path.join(
            os.environ['UBIQUITY_GLADE'], 'stepProtocol.ui'))
        builder.connect_signals(self)
        self.page = builder.get_object('stepProtocol')
        self.label1 = builder.get_object('label1')
        self.agree = builder.get_object('agree')
        self.plugin_widgets = self.page
        self.frontend = self.controller._wizard

    def on_agree_clicked(self, widget):
        if widget.get_active():
            self.frontend.next.set_sensitive(True)
        else:
            self.frontend.next.set_sensitive(False)

    def plugin_translate(self, lang):
        with open('/etc/linuxmint/eula') as f:
            text = f.read()
            self.label1.set_markup(text)

    def plugin_on_help_clicked(self):
        self.show_help('/usr/share/ubiquity/gtk/stepHelp.ui', self.help_dialog, self.help_label, self.help_prompt)

class Page(plugin.Plugin):
    def prepare(self):
        command = ['/usr/share/ubiquity/simple-plugins', 'protocol']
        questions = ['ubiquity/agree_protocol'] 
        self.ui.frontend.next.set_sensitive(False)
        return command, questions

    def run(self, priority, question):
        if (self.db.get('ubiquity/skip_eula') == 'true'):
            return True
        return plugin.Plugin.run(self, priority, question)

    def ok_handler(self):
        self.preseed_bool('ubiquity/agree_protocol', True)
        plugin.Plugin.ok_handler(self)
